#include"NMEA0183_GGA.h"
#include"NMEA0183_Semantik.h"
#include<ace/OS_NS_time.h>

NMEA0183_GGA::NMEA0183_GGA(NMEA0183_Message_Block *mb):NMEA0183_Message_Block(mb)
{
}

void
NMEA0183_GGA::CallVisitor(RACE_MessageVisitor *visitor)
{
    NMEA0183_Semantik *sem = dynamic_cast<NMEA0183_Semantik *>(visitor);
    if(sem)
    {
	sem->common(this);
	sem->GGA(this);
    }
}

ACE_Time_Value
NMEA0183_GGA::getUTCTimeOfFix() throw (NMEA0183_Protocol_Error)
{
    return getTimefromHHMMSS(1);
}

Angle_inDeg 
NMEA0183_GGA::getLatitude() throw (NMEA0183_Protocol_Error)
{
     return getAngleField(2);
}

bool 
NMEA0183_GGA::IsNorth() throw (NMEA0183_Protocol_Error)
{
    return getBooleanField('N','S',3);
}

Angle_inDeg 
NMEA0183_GGA::getLongitude() throw (NMEA0183_Protocol_Error)
{
    return getAngleField(4);
}

bool   
NMEA0183_GGA::IsEast() throw (NMEA0183_Protocol_Error)
{
    return getBooleanField('E','W',5);
}

NMEA0183_GGA::FixQuality NMEA0183_GGA::getFixQuality() throw (NMEA0183_Protocol_Error)
{
    int r=getIntField(6);
    switch(r)
    {
	case 0: return FQ_invalid;
	case 1: return FQ_GPS;
	case 2: return FQ_DGPS;
	default:
	    throw NMEA0183_Protocol_Error(this,"NMEA0183_GGA::getFixQuality: Illegal value %d as Fix Quality",r);
    }
    return FQ_invalid;
}

int NMEA0183_GGA::getNumberOfSatellitesBeeingTracked() throw (NMEA0183_Protocol_Error)
{
    return getIntField(7);
}

double NMEA0183_GGA::getHorizontalDilutionOfPosition() throw (NMEA0183_Protocol_Error)
{
    return getDoubleField(8);
}

double NMEA0183_GGA::getAltitudeAboveMeanSeaLevel_inM() throw (NMEA0183_Protocol_Error)
{
    checkfield("M",10);
    return getDoubleField(9);
}

double NMEA0183_GGA::getHightOfGeoidAboveWGS84Ellipsoid_inM() throw (NMEA0183_Protocol_Error)
{
    checkfield("M",12);
    return getDoubleField(11);
}

bool NMEA0183_GGA::IsDataGood()  throw (NMEA0183_Protocol_Error)
{
    return getFixQuality()!=FQ_invalid;
}
