#ifndef NMEA0183_GGA_H
#define NMEA0183_GGA_H
#include"NMEA0183_Message_Block.h"
#include"GeographicPositionData.h"

class NMEA0183_GGA : public NMEA0183_Message_Block, public GeographicPositionData
{
public:
    NMEA0183_GGA(NMEA0183_Message_Block *nmb);

    //##Documentation
    //## calls a NMEA0183_Semantik after the Visitor/Semantik-Pattern
    void CallVisitor(RACE_MessageVisitor *visitor);

     //##Documentation
    //## Fields 1 and current date
    ACE_Time_Value getUTCTimeOfFix() throw (NMEA0183_Protocol_Error);

    //##Documentation
    //## field 2
    Angle_inDeg getLatitude() throw (NMEA0183_Protocol_Error);

    //## true if Latitude in northern sphere
    //## if field 3 is 'N'
    bool   IsNorth() throw (NMEA0183_Protocol_Error);

    //##Documentation
    //## field 
    Angle_inDeg getLongitude() throw (NMEA0183_Protocol_Error);

    //##Documentation
    //## true if Longitude in ethern sphere
    //## if filed 4 is 'E'
    bool IsEast() throw (NMEA0183_Protocol_Error);

    enum FixQuality
    {
	FQ_invalid=0,
	FQ_GPS=1,
	FQ_DGPS=2
    };
    //##Documentation
    //## Field 5
    FixQuality getFixQuality() throw (NMEA0183_Protocol_Error);

    //##Documentation
    //## Field 6
    int getNumberOfSatellitesBeeingTracked() throw (NMEA0183_Protocol_Error);

    //##Documentation
    //## Field 7
    double getHorizontalDilutionOfPosition() throw (NMEA0183_Protocol_Error);

    //##Documentation
    //## Field 8 (Field 9 must be M)
    double getAltitudeAboveMeanSeaLevel_inM() throw (NMEA0183_Protocol_Error);

    //##Documentation
    //## Field 10 (Field 11 must be M)
    double getHightOfGeoidAboveWGS84Ellipsoid_inM() throw (NMEA0183_Protocol_Error);
    
    //##Documentation
    //## getFixQuality()!=FQ_invalid
    //## (may be good to have getNumberOfSatelitesBeeingTracked() > 3 or
    //## another  criteria
    bool IsDataGood() throw (NMEA0183_Protocol_Error);

};
#endif
