#include"NMEA0183_GLL.h"
#include"NMEA0183_Semantik.h"
#include<ace/Date_Time.h>

NMEA0183_GLL::NMEA0183_GLL(NMEA0183_Message_Block *mb):NMEA0183_Message_Block(mb)
{
}

void
NMEA0183_GLL::CallVisitor(RACE_MessageVisitor *visitor)
{
    NMEA0183_Semantik *sem = dynamic_cast<NMEA0183_Semantik *>(visitor);
    if(sem)
    {
	sem->common(this);
	sem->GLL(this);
    }
}

Angle_inDeg
NMEA0183_GLL::getLatitude() throw (NMEA0183_Protocol_Error)
{
    return getAngleField(1);
}

bool 
NMEA0183_GLL::IsNorth() throw (NMEA0183_Protocol_Error)
{
    return getBooleanField('N','S',2);
}

Angle_inDeg
NMEA0183_GLL::getLongitude() throw (NMEA0183_Protocol_Error)
{
    return getAngleField(3);
}

bool   
NMEA0183_GLL::IsEast() throw (NMEA0183_Protocol_Error)
{
    return getBooleanField('E','W',4);
}

bool  NMEA0183_GLL::IsDataGood() throw (NMEA0183_Protocol_Error)
{
    return getBooleanField('A','V',6);
}

ACE_Time_Value NMEA0183_GLL::getUTCTimeOfFix() throw (NMEA0183_Protocol_Error)
{
    return getTimefromHHMMSS(5);
}

NMEA0183_GLL::NMEA0183_GLL(Angle_inDeg latitude,bool north,
		 Angle_inDeg longitude,bool east,
		 ACE_Time_Value timeoffix,
			   bool dataisgood)
    :NMEA0183_Message_Block(256)
{
    ACE_Date_Time  date(timeoffix);
    set("GPGLL,%.0f%f,%c,%.0f%f,%c,%02ld%02ld%02ld,%c\r\n",
	latitude.deg,latitude.min,(north ? 'N' : 'S'),
	longitude.deg,longitude.min,(east ? 'E' : 'W'),
	date.hour(),date.minute(),date.second(),
	(dataisgood ? 'A':'V')
	);
}
