#include"NMEA0183_GSA.h"
#include"NMEA0183_Semantik.h"

NMEA0183_GSA::NMEA0183_GSA(NMEA0183_Message_Block *mb):NMEA0183_Message_Block(mb)
{
}

void
NMEA0183_GSA::CallVisitor(RACE_MessageVisitor *visitor)
{
    NMEA0183_Semantik *sem = dynamic_cast<NMEA0183_Semantik *>(visitor);
    if(sem)
    {
	sem->common(this);
	sem->GSA(this);
    }
}

bool 
NMEA0183_GSA::isSelectionAutomatic() throw (NMEA0183_Protocol_Error)
{
    return getBooleanField('A','M',1);
}

int 
NMEA0183_GSA::getDimensionMode()  throw (NMEA0183_Protocol_Error)
{
    return getIntField(2);
}

std::string 
NMEA0183_GSA::getSatelliteID(size_t index) throw (NMEA0183_Protocol_Error)
{
    if(1<=index && index<=12 && index+2<m_fields.size())
    {
	return getStringField(index+2);
    }
    else
    {
	if(index+2<m_fields.size())
	{
	    throw NMEA0183_Protocol_Error(this,"getSatelliteID: Index %d out of range 1 .. 12\n",index);
	}
	else
	{
	    throw NMEA0183_Protocol_Error(this,"getSatelliteID: Index %d out of range: Message may be broken: only %d Satellites available",index,m_fields.size()-2);
	}
    }
}

int 
NMEA0183_GSA::getNumKnownSatellites()
{
    int count=0;
    size_t i=3;
    while(i<=14 && i< m_fields.size())
    {
	if(m_fields[i].getLength()>0)
	{
	    count++;
	}
	i++;
    }
    return count;
}

double
NMEA0183_GSA::getPDOP_in_m() throw (NMEA0183_Protocol_Error)
{
    return getDoubleField(15);
}

double
NMEA0183_GSA::getHDOP_in_m() throw (NMEA0183_Protocol_Error)
{
    return getDoubleField(16);
}

double
NMEA0183_GSA::getVDOP_in_m() throw (NMEA0183_Protocol_Error)
{
    return getDoubleField(17);
}
