#include"NMEA0183_Message_Assembler.h"
#include"NMEA0183_Message_Block.h"

static envdebug debug("RACE_Message_Assembler");

//##ModelId=428354D50217
NMEA0183_Message_Assembler::NMEA0183_Message_Assembler(RACE::Message_Observer *processor)
    : RACE::Message_Assembler()
{
    statemashine_=new NMEA0183_Message_Assembler::StateMashine(processor);
    RACE_MEMORY_FAIL(statemashine_==0,__FILE__,__LINE__,__FUNCTION__);
    statemashine_->init();
}

/*---------------- State Mashine----------*/
//##ModelId=428354D501E7
NMEA0183_Message_Assembler::StateMashine::StateMashine(RACE::Message_Observer *processor)
    : RACE::Message_Assembler::StateMashine(),
      waitforbegin_(0),endfield_(0),waitforlf_(0)
{
    processor_=processor;
}

//##ModelId=428354D501F2
void NMEA0183_Message_Assembler::StateMashine::allocateMessageBlock()
{
    if(currentblock_==0)
    {
	currentblock_ = new NMEA0183_Message_Block(1024);
	RACE_MEMORY_FAIL(currentblock_==0,__FILE__,__LINE__,__FUNCTION__);
    }
}

//##ModelId=428354D501F8
NMEA0183_Message_Assembler::WaitForMessageBeginState::WaitForMessageBeginState(StateMashine *sm)
    :State(sm,"WaitForMessageBegin")
{
    
}

//##ModelId=428354D501E9
RACE::Message_Assembler::State *NMEA0183_Message_Assembler::StateMashine::getWaitForMessageBeginState()
{
    if(waitforbegin_==0)
    {
	waitforbegin_=new WaitForMessageBeginState(this);
	RACE_MEMORY_FAIL(waitforbegin_==0,__FILE__,__LINE__,__FUNCTION__);
    }
    return waitforbegin_;
}

RACE::Message_Assembler::State *
NMEA0183_Message_Assembler::StateMashine::getEndFieldState()
{
    if(endfield_==0)
    {
	endfield_=new EndFieldState(this);
	RACE_MEMORY_FAIL(endfield_==0,__FILE__,__LINE__,__FUNCTION__);
    }
    return endfield_;
}

//##ModelId=428354D501EF
RACE::Message_Assembler::State *NMEA0183_Message_Assembler::StateMashine::getWaitForLFState()
{
    if(waitforlf_==0)
    {
	waitforlf_=new WaitForLFState(this);
	RACE_MEMORY_FAIL(waitforlf_==0,__FILE__,__LINE__,__FUNCTION__);
    }
    return waitforlf_;
}


//##ModelId=428354D501EE
RACE::Message_Assembler::State *NMEA0183_Message_Assembler::StateMashine::getCollectMessageState()
{
    if(collectmsg_state_==0)
    {
	collectmsg_state_=new NMEA0183_Message_Assembler::CollectMessageState(this);
	RACE_MEMORY_FAIL(collectmsg_state_==0,__FILE__,__LINE__,__FUNCTION__);
    }
    return collectmsg_state_;
}

//##ModelId=428354D501F0
RACE::Message_Assembler::State *NMEA0183_Message_Assembler::StateMashine::getBeginReadingState()
{
    return getWaitForMessageBeginState();
}

//##ModelId=428354D501FD
void NMEA0183_Message_Assembler::WaitForMessageBeginState::enter(char)
{
    debug.print("Enter State WaitForMessageBeginState");
    sm_->allocateMessageBlock();
}

//##ModelId=428354D501FA
void NMEA0183_Message_Assembler::WaitForMessageBeginState::consume(char c)
{
    NMEA0183_Message_Assembler::StateMashine *ksm=ACE_dynamic_cast(NMEA0183_Message_Assembler::StateMashine *,sm_);
    RACE_COND_FAIL(ksm==0,"Can't narrow ro NMEA0183_Message_Assembler::StateMashine\n");
    debug.print("WaitForMessageBeginState: <- %c",c);
    if(c=='$')
    {
	*(sm_->currentblock()->wr_ptr(1))=c;
	ACE_dynamic_cast(EndFieldState *,ksm->getEndFieldState())->setbegin(sm_->currentblock()->wr_rel());
	ksm->changeState(sm_->getCollectMessageState(),c);
    }
    else
    {
	debug.print("WaitForMessageBeginState: ignore '%c'",c);
    }
}

/* CollectMessageState */
//##ModelId=428354D50202
NMEA0183_Message_Assembler::CollectMessageState::CollectMessageState(StateMashine *sm)
    :State(sm,"CollectMessage")
{
    
}

//##ModelId=428354D50207
void NMEA0183_Message_Assembler::CollectMessageState::enter(char /*c*/)
{
    debug.print("Enter State CollectMessageState");
}

//##ModelId=428354D50204
void NMEA0183_Message_Assembler::CollectMessageState::consume(char c)
{
    debug.print("CollectMessageState: <- %c",c);
    NMEA0183_Message_Assembler::StateMashine *ksm=ACE_dynamic_cast(NMEA0183_Message_Assembler::StateMashine *,sm_);
    RACE_COND_FAIL(ksm==0,"%s:%d: Error in casting to konkrete State Machine ",__FILE__,__LINE__);

    switch(c)
    {
    case ',':
    case '*':
    case '\n':
    case '\r':
	  *(sm_->currentblock()->wr_ptr(1))=c;
	  sm_->changeState(ksm->getEndFieldState(),c);
	  break;
    case '$':
	sm_->changeState(ksm->getEndFieldState(),c);
	break;
    default:
	 *(sm_->currentblock()->wr_ptr(1))=c;
	break;
    };
}

/*---------- EndField State ------------*/
NMEA0183_Message_Assembler::EndFieldState::EndFieldState(StateMashine *sm)
    :State(sm,"WaitForLFState"),begin_(0)
{
    
}

void
NMEA0183_Message_Assembler::EndFieldState::setbegin(size_t pos)
{
    begin_=pos;
}

void
NMEA0183_Message_Assembler::EndFieldState::enter(char c)
{
    NMEA0183_Message_Block *cur =
	dynamic_cast<NMEA0183_Message_Block *>(sm_->currentblock());
    RACE_COND_FAIL(cur==0,"NMEA0183_Message_Assembler: Can't narrow to an NMEA0183_Message_Block");
    NMEA0183_Message_Assembler::StateMashine *ksm=ACE_dynamic_cast(NMEA0183_Message_Assembler::StateMashine *,sm_);
    RACE_COND_FAIL(ksm==0,"Can't narrow to an NMEA0183_Message_Assembler::StateMashine");
    size_t end= cur->wr_rel()-1; // current write position is behind the comma
    size_t flen=end - begin_;
    debug.print("Enter EndFieldState: Field(%u,%u)\n",begin_,flen);
    cur->add(NMEA0183_Message_Block::Field(begin_,flen));
    begin_=cur->wr_rel(); // current write position is beginn of the field
    switch(c)
    {
	case '*':
	    cur->nextFieldIsCheckSumField();
	    break;
	case '\n':
	    sm_->changeState(sm_->getSendMessageState(),c);
	    break;
	case '\r':
	    sm_->changeState(ksm->getWaitForLFState(),c);
 	    break;
	case '$':
	    RACE_ERROR("CollectMessageState:: Unexpected '$'"
		       " - Begin new message \n");
	    //Should be deleted in the MessageBuffer ?
	    sm_->changeState(ksm->getSendMessageState(),c);
	    sm_->consume(c);
	    break;
	default: break;
    }
}

void
NMEA0183_Message_Assembler::EndFieldState::consume(char c)
{
    sm_->changeState(sm_->getCollectMessageState(),'\0');
    sm_->getCurrentState()->consume(c);
}

/*--------- WaitForLFState -------------*/
//##ModelId=428354D5020C
NMEA0183_Message_Assembler::WaitForLFState::WaitForLFState(StateMashine *sm)
    :State(sm,"WaitForLFState")
{
    
}

//##ModelId=428354D50211
void NMEA0183_Message_Assembler::WaitForLFState::enter(char /*c*/)
{
    debug.print("Enter State WaitForLFState");
}

//##ModelId=428354D5020E
void NMEA0183_Message_Assembler::WaitForLFState::consume(char c)
{
    debug.print("WaitForLFState: <- %c",c);
    switch(c)
    {
	case '\n':
	    *(sm_->currentblock()->wr_ptr(1))=c;
	    sm_->changeState(sm_->getSendMessageState(),c);
	    break;	
	case '$':
	    RACE_ERROR("WaitForLFState: Missing CR \n");
	    *(sm_->currentblock()->wr_ptr(1))='\n';
	    //Shortcut 
	    sm_->changeState(sm_->getSendMessageState(),c);
	    //sm_->consume('$');
	    break;
	default: 
	    RACE_ERROR("WaitForLFState:: Unexpected '%c=%d' \n",c,c);
    }
}
