#include"NMEA0183_Message_Block.h"
#include"NMEA0183_Semantik.h"
#include<locale.h>
#include<stdlib.h>
#include<errorhandling/envdebug.h>
#include<ace/OS_NS_time.h>
#include"NumFieldParsing.h"

//##ModelId=428354D5022C
NMEA0183_Message_Block::Field::Field():m_offset(0),m_len(0)
{
}

//##ModelId=428354D50236
NMEA0183_Message_Block::Field::Field(off_t off,size_t len):m_offset(off),m_len(len)
{
}

//##ModelId=428354D50234
NMEA0183_Message_Block::Field::Field(const Field& other)
    :m_offset(other.m_offset),
     m_len(other.m_len)
{
}

//##ModelId=428354D50239
off_t NMEA0183_Message_Block::Field::getOffset()
{
    return m_offset;
}

//##ModelId=428354D5023A
size_t NMEA0183_Message_Block::Field::getLength()
{
    return m_len;
}

//##ModelId=428354D50248
NMEA0183_Message_Block::NMEA0183_Message_Block(size_t len):RACE_Message_Block(len)
{
    m_chksum_idx=0;
    /*
     * the locale is used to set country specific settings.
     * The NMEA0183 is independed of that, and we don't want to get surprises
     * Thus we set the expicitly the POSIX locale here.
     */
    RACE_COND_FAIL(setlocale(LC_NUMERIC,"POSIX")==NULL,"Can't set POSIX-Locale\n");
}

//##ModelId=428354D5024A
NMEA0183_Message_Block::NMEA0183_Message_Block(const char *str):RACE_Message_Block(strlen(str)+1)
{
    copy(str,strlen(str));
    m_chksum_idx=0;
}

//##ModelId=428354D5024C
NMEA0183_Message_Block::NMEA0183_Message_Block(NMEA0183_Message_Block *other)
    :RACE_Message_Block(other->duplicate_chain(),other->IsFragmented())
{
    for(size_t i=0;i<other->m_fields.size();i++)
    {
	m_fields.push_back(other->m_fields[i]);
    }
    m_chksum_idx=other->m_chksum_idx;
}

//##ModelId=428354D50255
void  NMEA0183_Message_Block::add(const NMEA0183_Message_Block::Field &f)
{
    m_fields.push_back(f);
}

void
NMEA0183_Message_Block::nextFieldIsCheckSumField()
{
  m_chksum_idx=m_fields.size();  
}

//##ModelId=428354D50254
bool NMEA0183_Message_Block::parse()
{
    // Assume Message is in one RACE_Data_Block
    RACE_COND_FAIL(IsFragmented(),"%s:%d: Message Block is fragmented",__FILE__,__LINE__);
    off_t fieldbegin;
    size_t fieldlen;
    char ch;
    if(*rd_ptr(1)!='$') return false;
    ch=*rd_ptr();
    while(remaining()>0 && ch!='\r' && ch!='\n')
    {
	fieldbegin=rd_rel();
	fieldlen=0;
	while(remaining()>0 && (ch=*rd_ptr(1))!=',' && ch!='*' && ch!='\r' && ch!='\n')
	{
	    fieldlen++;
	}
	// even zero length fields are permitted
	add(Field(fieldbegin,fieldlen));
	if(ch=='*')
	{
	    m_chksum_idx=m_fields.size();
	}
    }
    return true;
}

std::string  NMEA0183_Message_Block::getCheckSum() throw (NMEA0183_Protocol_Error)
{
    if(m_chksum_idx<=0)
    {
	throw NMEA0183_Protocol_Error(this,"Checksum isn't avialable");
    }
    return getStringField(m_chksum_idx);
}

//##ModelId=428354D50270
std::string NMEA0183_Message_Block::calcCheckSum()
{
    // Assume Message is in one RACE_Data_Block
    RACE_COND_FAIL(IsFragmented(),"%s:%d: Message Block is fragmented",__FILE__,__LINE__);
    unsigned char sum=0;
    char erg[3];
    RACE_COND_FAIL(*base()!='$',"illegal Message");
    for(char *i=base()+1;i<base()+length() && *i!='*' && *i!='\r' && *i!='\n';i++)
    {
	sum^= (unsigned char)(*i);
    }           
    snprintf(erg,3,"%02X",(int)sum);
    return std::string(erg,2);
}

//##ModelId=42EF6192034C
void 
NMEA0183_Message_Block::setCheckSum()
{
    std::string csum=calcCheckSum();
    // positioniere auf ende
    char *i;
    for(i=base();i<base()+length() && *i!='*' && *i!='\r' && *i!='\n';i++); 
    wr_ptr(i);
    *wr_ptr(1)='*';
    copy(csum.c_str(),2);
    copy("\r\n",2);
}

//##ModelId=428354D5026A
size_t NMEA0183_Message_Block::getNumberOfFields()
{
    return m_fields.size();
}

//##ModelId=428354D5025D
std::string NMEA0183_Message_Block::getStringField(size_t fieldno) throw (NMEA0183_Protocol_Error)
{
    if(fieldno>=getNumberOfFields())
    {
	throw NMEA0183_Protocol_Error(this,"NMEA0183_Message_Block:getField: illegal field number %d (max %d)",fieldno,getNumberOfFields());
    }
    Field& f =m_fields[fieldno];
    return std::string(base()+f.getOffset(),f.getLength());
}

//##ModelId=428354D5025F
double NMEA0183_Message_Block::getDoubleField(size_t fieldno) throw (NMEA0183_Protocol_Error)
{
   if(fieldno>=getNumberOfFields())
   {
       throw NMEA0183_Protocol_Error(this,"NMEA0183_Message_Block:getField: illegal field number %d (max %d)",fieldno,getNumberOfFields());
   }
   Field& f =m_fields[fieldno];
   char *e;
   double ret=strtod(base()+f.getOffset(),&e);
   if(*e!=','&&*e!='*')
   {
       throw  NMEA0183_Protocol_Error(this,"Can't convert field %d to double",
				       fieldno);
   }
   if(*e!=','&&*e!='*')
   {
       throw NMEA0183_Protocol_Error(this,"%s:%d: Conversion Error ",__FILE__,__LINE__);
   }
   return ret;
}

//##ModelId=42B8090D0365
int NMEA0183_Message_Block::getIntField(size_t fieldno) throw (NMEA0183_Protocol_Error)
{
   if(fieldno>=getNumberOfFields())
   {
       throw NMEA0183_Protocol_Error(this,"NMEA0183_Message_Block:getField: illegal field number %d (max %d)",fieldno,getNumberOfFields());
   }
   Field& f =m_fields[fieldno];
   char *e;
   int ret=strtol(base()+f.getOffset(),&e,10);
   if(*e!=','&&*e!='*')
   {
       throw  NMEA0183_Protocol_Error(this,"Can't convert field %d to integer",
				       fieldno);
   }
   return ret;
}

//##ModelId=428354D50266
bool NMEA0183_Message_Block::getBooleanField(char y,char n,size_t fieldno) throw (NMEA0183_Protocol_Error)
{
   if(fieldno>=getNumberOfFields())
   {
       throw NMEA0183_Protocol_Error(this,"NMEA0183_Message_Block:getField: illegal field number %d (max %d)",fieldno,getNumberOfFields());
   }
   Field& f =m_fields[fieldno];
   if(f.getLength()!=1)
   {
      throw  NMEA0183_Protocol_Error(this,"Field %d to long for BooleanField",
				      fieldno);
   }
   char b=*(base()+f.getOffset());
   if(b==y) return true;
   else if(b==n) return false;
   else 
   {
       throw NMEA0183_Protocol_Error(this,
			  "In Field %d %c or %c expected, %c received",
			  fieldno,y,n,b);
       return false;
   }
}

//##ModelId=428354D5025C
std::string NMEA0183_Message_Block::getType()
{
    return getStringField(0);
}

std::string
NMEA0183_Message_Block::getTalkerID()
{
   Field& f =m_fields[0];

   return std::string(base()+f.getOffset(),2);
}

std::string
NMEA0183_Message_Block::getSentenceID()
{
    Field& f =m_fields[0];
    return std::string(base()+f.getOffset()+2,f.getLength()-2);
}

void
NMEA0183_Message_Block::checkfield(const char *shall,size_t fieldno) throw (NMEA0183_Protocol_Error)
{
    if(fieldno>=getNumberOfFields())
    {
	throw NMEA0183_Protocol_Error(this,"NMEA0183_Message_Block:getField: illegal field number %d (max %d)",fieldno,getNumberOfFields());
    }
    Field& f = m_fields[fieldno];
    const char *p=shall;
    const char *pf=base()+f.getOffset();
    size_t i=0;
    while(*p==*pf && i<f.getLength() && *p!='\0')
    {
	p++;pf++;i++;
    } 
    if(i<f.getLength() || *p!='\0')
    {
	throw NMEA0183_Protocol_Error(this,"Field %d doesn't match %s",
				       fieldno,shall
				       );
    }
}

Angle_inDeg
NMEA0183_Message_Block::getAngleField(size_t fieldno)
{
    Angle_inDeg ret;
    Field &angle = m_fields[fieldno];
    char *astr= base()+angle.getOffset();
    size_t i=0;
    size_t consumed=0;
    while(i<angle.getLength() && astr[i]!='.')
    {
	i++;
    }
    ret.deg=(double)NumFieldParsing::getLong(astr,i-2,consumed);
    ret.min=NumFieldParsing::getFixPoint(astr+i-2,angle.getLength()-(i-2));
    return ret;
}

void 
NMEA0183_Message_Block::getTimefromField(struct tm& erg,double& fracerg,size_t fnum) throw (NMEA0183_Protocol_Error)
{
    size_t consumed;
    Field &timef = m_fields[fnum]; // hhmmss[.ss]
    if(timef.getLength()<6)
    {
	throw NMEA0183_Protocol_Error(this,"Length of Field fnum is < 6 ");
    }
    char *timestr = base()+timef.getOffset();
    erg.tm_hour= NumFieldParsing::getLong(timestr,2,consumed);
    erg.tm_min = NumFieldParsing::getLong(timestr+2,2,consumed);
    erg.tm_sec = NumFieldParsing::getLong(timestr+4,2,consumed);

    fracerg=0.0;
    size_t remaining=timef.getLength()-6;
    if(remaining>1 && timestr[6]=='.')
    {
	long fracseconds=0;
	double quot=1.0;
	remaining--;
	size_t i=7;
	while(remaining>0)
	{
	    fracseconds *= 10;
	    fracseconds += (timestr[i]-'0');
	    quot*=10.0;
	    i++;
	    remaining--;
	}
	fracerg=(double)fracseconds/quot;
    }
}

void 
NMEA0183_Message_Block::getDatefromField(struct tm& erg,size_t fnum) throw (NMEA0183_Protocol_Error)
{
    if(fnum>=getNumberOfFields())
    {
	throw NMEA0183_Protocol_Error(this,"NMEA0183_Message_Block:getField: illegal field number %d (max %d)",fnum,getNumberOfFields());
    }
    Field &datef = m_fields[fnum]; // ddmmyy
    size_t consumed;
    
    if(datef.getLength()!=6)
    {
	throw NMEA0183_Protocol_Error(this,"Length of Field %d is not 6 ",fnum);
    }
    char *datestr = base()+datef.getOffset();
    erg.tm_mday=  NumFieldParsing::getLong(datestr,2,consumed);
    erg.tm_mon=   NumFieldParsing::getLong(datestr+2,2,consumed);
    erg.tm_mon -=1; // Range from 0 .. 11
    erg.tm_year=  NumFieldParsing::getLong(datestr+4,2,consumed);
    if(erg.tm_year<50) erg.tm_year+=100;
}

ACE_Time_Value NMEA0183_Message_Block::getTimefromHHMMSS(size_t fnum) throw (NMEA0183_Protocol_Error)
{
    if(fnum>=getNumberOfFields())
    {
	throw NMEA0183_Protocol_Error(this,"NMEA0183_Message_Block:getField: illegal field number %d (max %d)",fnum,getNumberOfFields());
    }
    struct tm datetime;
    ACE_Time_Value curtime=ACE_OS::gettimeofday();
    time_t gmtime = curtime.sec();
    ACE_OS::gmtime_r(&gmtime,&datetime);
    double fracseconds=0.0;
    getTimefromField(datetime,fracseconds,fnum);
    datetime.tm_gmtoff=0;
    datetime.tm_zone=0;
    datetime.tm_isdst=-1;
    ACE_Time_Value ret(ACE_OS::mktime(&datetime),0);
    ACE_Time_Value fractv;
    fractv.set(fracseconds);
    ret+=fractv;
    return ret;
}

//##ModelId=428354D50252
void NMEA0183_Message_Block::CallVisitor(RACE_MessageVisitor *visitor)
{
    NMEA0183_Semantik *sem = dynamic_cast<NMEA0183_Semantik *>(visitor);
    if(sem)
    {
	sem->UnknownMessage(this);
    }
}

void NMEA0183_Message_Block::set(const char *msg,...) //__attribute__ ((format(printf,2,3)))
{
    va_list args;
    va_start(args,msg);
    reset();
    *(wr_ptr(1))='$';
    int l=vsnprintf(wr_ptr(),freespace(),msg,args);
    va_end(args);
    wr_ptr(l);
}

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Vector<Field,32>;
#elif defined (ACE_HAS_EXPLICIT_STATIC_TEMPLATE_MEMBER_INSTANTIATION)
#pragma instantiate class ACE_Vector<Field,32> 
#endif

//##ModelId=4291BA20000B
NMEA0183_Protocol_Error::NMEA0183_Protocol_Error(NMEA0183_Message_Block *block,const char *msg,...):m_block(block)
{
  va_list args;
  va_start(args,msg);
  vsnprintf(m_errormsg,sizeof(m_errormsg),msg,args);
  va_end(args);
  m_block->add_ref();
}

//##ModelId=4291BA20000F
NMEA0183_Protocol_Error::NMEA0183_Protocol_Error(const NMEA0183_Protocol_Error& other)
    : std::exception(other)
{
    strcpy(m_errormsg,other.m_errormsg);
    m_block=other.m_block;
    m_block->add_ref();
}

//##ModelId=4291BA200020
NMEA0183_Protocol_Error::~NMEA0183_Protocol_Error() throw ()
{
    m_block->release();
}

//##ModelId=4291BA200022
const char *
NMEA0183_Protocol_Error::what() const throw ()
{
    return m_errormsg;
}

//##ModelId=4291BA200029
NMEA0183_Message_Block *
NMEA0183_Protocol_Error::getMessageBlock()
{
    return m_block;
}
