#ifndef NMEA0183_SEMANTIK_H
#define NMEA0183_SEMANTIK_H
#include<RACE_Message_Block.h>
#include"GeographicPositionData.h"
#include"VectorSpeedData.h"

class NMEA0183_Message_Block;
class NMEA0183_RMC;
class NMEA0183_GLL;
class NMEA0183_VTG;
class NMEA0183_GGA;

class NMEA0183_GSA;
class NMEA0183_GSV;

//##ModelId=428354D5000E
//##Documentation
//## This interface abstracts from the using of a NMEA0183_Message_Block.
//## 
//## Note that we have a method "common" which allow a general handling of a
//## NMEA0183 sentence and
//## a specific handling (the name of the method is the name of the sentence).
//## The default semantic of the common-Method is to check if the Datablock is
//## a GeographicPositionData 
//## or an VectorSpeedData and call the abstract semantic methods for these
//## abstract data
class NMEA0183_Semantik : public RACE_MessageVisitor
{
 public:
    //##ModelId=428354D502CC
    //##Documentation
    //## returns name of the semantic
    //## - mainly for allowing the general RACE_MessageVisitor
    virtual const char *name();
    //##ModelId=428354D502CE
    //##Documentation
    //## This is called from NMEA0183_Message_Block and means, that the system
    //## don't know this sentence type
    virtual void UnknownMessage(NMEA0183_Message_Block *mb)=0;
    
    //##ModelId=428354D502D7
    //##Documentation
    //## This is called first and is for general operations on the
    //## NMEA0183_Message_Block.
    //## The default handling is to check if the NMEA0183 Message_Block is of
    //## an abstract kind
    //## (GeographicPositionData or VectorSpeedData) and call the corresponded
    //## methods)
    virtual void common(NMEA0183_Message_Block *mb);
    //##ModelId=42B8090E0009
    //##Documentation
    //## Handels the GeographicPositionData
    virtual void GeographicPosition(GeographicPositionData *gp)=0;
    //##ModelId=42B8090E0018
    //##Documentation
    //## Handels the VectorSpeedData
    virtual void VectorSpeed(VectorSpeedData *vs)=0;
    //##ModelId=42B8090E001B
    //##Documentation
    //## Handels (the specialities) of a NMEA0193_RMC-Sentenence
    virtual void RMC(NMEA0183_RMC *rmc)=0;
    //##ModelId=42B8090E0027
    //##Documentation
    //## Handeles the specialities of an NMEA0183_GLL
    virtual void GLL(NMEA0183_GLL *gll)=0;
    //##ModelId=42B8090E0037
    //##Documentation
    //## Handeles the specialites of an NMEA0183_GGA
    virtual void GGA(NMEA0183_GGA *gga)=0;
    //##ModelId=42B8090E003A
    //##Documentation
    //## Handles the specialities of an NMEA083_VTG 
    virtual void VTG(NMEA0183_VTG *vtg)=0;

    //##Documentation
    //## Handles the specialities of an NMEA083_GSA 
    virtual void GSA(NMEA0183_GSA *gsa)=0;

    //##Documentation
    //## Handles the specialities of an NMEA083_GSV 
    virtual void GSV(NMEA0183_GSV *gsv)=0;

};
#endif
