// default semantik for NMEA0183
#include<ace/streams.h>
#include"NMEA0183_Semantik_Printout.h"
#include"NMEA0183_Message_Block.h"
#include"NMEA0183_RMC.h"
#include"NMEA0183_GLL.h"
#include"NMEA0183_VTG.h"
#include"NMEA0183_GGA.h"
#include"NMEA0183_GSA.h"
#include"NMEA0183_GSV.h"
#include<ace/Date_Time.h>

std::ostream& operator<<(std::ostream& str,Angle_inDeg angle)
{
    str << angle.deg << " Degree " << angle.min << " Minutes";
    return str;
}

//##ModelId=428354D502CC
const char *
NMEA0183_Semantik_Printout::name()
{
    return "Default-Printout";
}

NMEA0183_Semantik_Printout::NMEA0183_Semantik_Printout(bool verbose)
    :verbose_(verbose)
{

}
void
NMEA0183_Semantik_Printout::common(NMEA0183_Message_Block *nmb)
{
    if(verbose_)
    {
	std::cout << "----- Message Fields --------------" << std::endl;
	UnknownMessage(nmb);
    }
    NMEA0183_Semantik::common(nmb);
}

//##ModelId=428354D502CE
void
NMEA0183_Semantik_Printout::UnknownMessage(NMEA0183_Message_Block *nmb)
{
    std::cout << "Message has " << nmb->getNumberOfFields() << " Fields " << std::endl;
    std::cout << "Message is a " << nmb->getType()
	      << " Sender " << nmb->getTalkerID()
	      << " Sentence " << nmb->getSentenceID()
	      << std::endl;
    for(size_t i=0;i<nmb->getNumberOfFields();i++)
    {
	std::cout << i << "\t" << nmb->getStringField(i) << std::endl;
    }
    std::cout << "Calculated Checksum: " << nmb->calcCheckSum() << std::endl;
    try
    {
	std::cout << "Read Checksum: " << nmb->getCheckSum() << std::endl;
    }
    catch(...)
    {
    }
}


void NMEA0183_Semantik_Printout::GeographicPosition(GeographicPositionData *gpd)
{
    std::cout << "============  Geographic Data: ==========" << std::endl;
    try
    {
	std::cout << "Data is " << (gpd->IsDataGood() ? "good" : "bad" ) << std::endl;
	std::cout << "Latitude:\t" << gpd->getLatitude() << " " 
		  << (gpd->IsNorth() ? "North" : "South") << std::endl;
	std::cout << "Longitude:\t" << gpd->getLongitude() << " "
		  << (gpd->IsEast() ? "East" : "West") << std::endl;
	ACE_Time_Value utcfix=gpd->getUTCTimeOfFix();
	ACE_Date_Time  date(utcfix);
	std::cout << "Timevalue:(" << utcfix.sec() << ':' << utcfix.usec() << ") " << std::endl;
	std::cout << "Recalculate Date and Time:\t"
		  << date.day() << '.' << date.month() << '.' << date.year() << "  "
		  << date.hour() << ':' << date.minute() << ':' << date.second()
		  << std::endl;
    }
    catch(NMEA0183_Protocol_Error &ex)
    {
	ERRLOG << "Wrong Message: " << ex.what() << E_EOM;
    }
}

void NMEA0183_Semantik_Printout::VectorSpeed(VectorSpeedData *vs)
{
    std::cout << "========== Vector Speed data: =============" << std::endl;
    try
    {
	std::cout << "True Track made Good in Degrees :" 
		  << vs->getTrueTrackMadeGood_inDegrees() << std::endl;
	std::cout << "Ground Speed in Knots "
		  << vs->getGroundSpeed_inKnots() << std::endl;
    }
    catch(NMEA0183_Protocol_Error &ex)
    {
	ERRLOG << "Wrong Message: " << ex.what() << E_EOM;
    }
}

//##ModelId=428354D502D7
void 
NMEA0183_Semantik_Printout::RMC(NMEA0183_RMC *rmc)
{
    std::cout << "--- RMC - Recomended minimum specific GPS/Transit data ---" 
	      << std::endl;
    try
    {
	ACE_Time_Value utcfix=rmc->getUTCTimeOfFix();
	ACE_Date_Time  date(utcfix);
	std::cout << "Timevalue:(" << utcfix.sec() << ':' << utcfix.usec() << ") " << std::endl;
	std::cout << "Recalculate Date and Time:\t"
		  << date.day() << '.' << date.month() << '.' << date.year() << "  "
		  << date.hour() << ':' << date.minute() << ':' << date.second()
		  << std::endl;
	std::cout << "Data is " << (rmc->IsDataGood() ? "good" : "bad" ) << std::endl;
	std::cout << "Latitude:\t" << rmc->getLatitude() << " " 
		  << (rmc->IsNorth() ? "North" : "South") << std::endl;
	std::cout << "Longitude:\t" << rmc->getLongitude() << " "
		  << (rmc->IsEast() ? "East" : "West") << std::endl;
	std::cout << "Speed over Ground:\t" << rmc->getGroundSpeed_inKnots() << " knots" << std::endl;
	std::cout << "TrackMadeGood:\t" << rmc->getTrueTrackMadeGood_inDegrees() << " Degrees " << std::endl;
	std::cout << "MagneticVariation:\t" << rmc->getMagneticVariation_inDegrees() << " Degrees ";
	std::cout << (rmc->IsMagneticVariationEast() ? "East" : "West") << std::endl;
    }
    catch(NMEA0183_Protocol_Error &ex)
    {
	ERRLOG << "Wrong Message: " << ex.what() << E_EOM;
    }
}

void NMEA0183_Semantik_Printout::GLL(NMEA0183_GLL *gll)
{
    std::cout << "--- GLL -Geographic position, Latidude and Longitude -- "
	      << std::endl;
    try
    {
	std::cout << "Data is " << (gll->IsDataGood() ? "good" : "bad" ) << std::endl;
	std::cout << "Latitude:\t" << gll->getLatitude() << " " 
		  << (gll->IsNorth() ? "North" : "South") << std::endl;
	std::cout << "Longitude:\t" << gll->getLongitude() << " "
		  << (gll->IsEast() ? "East" : "West") << std::endl;
	ACE_Time_Value utcfix=gll->getUTCTimeOfFix();
	ACE_Date_Time  date(utcfix);
	std::cout << "Timevalue:(" << utcfix.sec() << ':' << utcfix.usec() << ") " << std::endl;
	std::cout << "Recalculate Date and Time:\t"
		  << date.day() << '.' << date.month() << '.' << date.year() << "  "
		  << date.hour() << ':' << date.minute() << ':' << date.second()
		  << std::endl;
    }
    catch(NMEA0183_Protocol_Error &ex)
    {
	ERRLOG << "Wrong Message: " << ex.what() << E_EOM;
    }
}

std::ostream& operator<<(std::ostream& ostr,NMEA0183_GGA::FixQuality fq)
{
    switch(fq)
    {
	case NMEA0183_GGA::FQ_invalid: ostr << "invalid"; break;
	case NMEA0183_GGA::FQ_GPS:     ostr << "GPS"; break;
	case NMEA0183_GGA::FQ_DGPS:    ostr << "DGPS"; break;
	default: break;
    }
    return ostr;
}

void NMEA0183_Semantik_Printout::GGA(NMEA0183_GGA *gga)
{
    std::cout << "--- GGA - Global Positioning System Fix Data --- " 
	      << std::endl;
    try
    {
	ACE_Time_Value utcfix=gga->getUTCTimeOfFix();
	ACE_Date_Time  date(utcfix);
	std::cout << "Timevalue:(" << utcfix.sec() << ':' << utcfix.usec() << ") " << std::endl;
	std::cout << "Recalculate Date and Time:\t"
		  << date.day() << '.' << date.month() << '.' << date.year() << "  "
		  << date.hour() << ':' << date.minute() << ':' << date.second()
		  << std::endl;
	std::cout << "Latitude:\t" << gga->getLatitude() << " " 
		  << (gga->IsNorth() ? "North" : "South") << std::endl;
	std::cout << "Longitude:\t" << gga->getLongitude() << " "
		  << (gga->IsEast() ? "East" : "West") << std::endl;
	
	std::cout << "FixQuality: " << gga->getFixQuality() << std::endl;
	std::cout << "Number of Satelites beeing tracked "
		  << gga->getNumberOfSatellitesBeeingTracked() << std::endl;
	std::cout << "HorizontalDilutionOfPosition: "
		  << gga->getHorizontalDilutionOfPosition() << std::endl;
	
	std::cout << "AltitudeAboveMeanSeaLevel_inM: "
		  << gga->getAltitudeAboveMeanSeaLevel_inM() << std::endl;
    }
    catch(NMEA0183_Protocol_Error &ex)
    {
	ERRLOG << "Wrong Message: " << ex.what() << E_EOM;
    }
}

void NMEA0183_Semantik_Printout::VTG(NMEA0183_VTG *vtg)
{
    std::cout << "--- VTG - Track made good and ground speed --- "
	      << std::endl;
    try
    {
	std::cout << "TrueTrackMadeGood_inDegrees "
		  << vtg->getTrueTrackMadeGood_inDegrees() << std::endl;
	std::cout << "MagneticTrackMadeGood_inDegrees() "
		  << vtg->getMagneticTrackMadeGood_inDegrees() << std::endl;
	std::cout << "GroundSpeed_inKnots " << vtg->getGroundSpeed_inKnots()
		  << std::endl;
	std::cout << "GroundSpeed_inKpH " << vtg->getGroundSpeed_inKpH()
		  << std::endl;
	
    }
    catch(NMEA0183_Protocol_Error &ex)
    {
	ERRLOG << "Wrong Message: " << ex.what() << E_EOM;
    }
}

void 
NMEA0183_Semantik_Printout::GSA(NMEA0183_GSA *gsa)
{
    std::cout << "--- GSA - GPS DOP and active satellites ---"
	      << std::endl;
    try
    {
	std::cout << "Selection of dimension mode " 
		  << (gsa->isSelectionAutomatic() ? "Automatic" : "manuall")
		  << std::endl;
	std::cout << "Dimensions: " << gsa->getDimensionMode() << std::endl;
	std::cout << gsa->getNumKnownSatellites() << " Satellites known "
		  << std::endl;
	for(size_t i=1;i<=gsa->getMaxSatellites();i++)
	{
	    std::cout << "No" << i << ": " << gsa->getSatelliteID(i)
		      << std::endl;
	}
	std::cout << "PDOP in m: " << gsa->getPDOP_in_m() << std::endl;
	std::cout << "VDOP in m: " << gsa->getVDOP_in_m() << std::endl;
	std::cout << "HDOP in m: " << gsa->getHDOP_in_m() << std::endl;
    }
    catch(NMEA0183_Protocol_Error &ex)
    {
	ERRLOG << "Wrong Message: " << ex.what() << E_EOM;
    }
}

void
NMEA0183_Semantik_Printout::GSV(NMEA0183_GSV *gsv)
{
    std::cout << "--- GSV - Satelites in View --- " << std::endl;
    try
    {
	std::cout << "Message " << gsv->getMessageNumber() << " of "
		  << gsv->getTotalNumber() << std::endl;
	std::cout << gsv->getSatellitesInView() << " Satellites in view" << std::endl;
	NMEA0183_GSV::SatData data;
	while(gsv->getNextSatelliteData(data))
	{
	    std::cout << "Satellite " << data.prn << ": "
		      << " Elevation " <<  data.elevation
		      << " Azimuth   " <<  data.azimuth
		      << " snr in dB " <<  data.snr
		      << std::endl;
	}
	gsv->resetSatelliteDataIterator();
    }
    catch(NMEA0183_Protocol_Error &ex)
    {
	ERRLOG << "Wrong Message: " << ex.what() << E_EOM;
    }
}
