#ifndef RACE_Device_H
#define RACE_Device_H

#include<errorhandling/errorhandling.h>
#include<errorhandling/envdebug.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>

class SystemCallInterrupted: public std::exception
{
public:
    SystemCallInterrupted(const char *systemcall);
    virtual const char *what() const throw();
private:
    char msg_[64];
};

//##Documentation
//## UNIX-specific simple OO-Wrapper for a Device like fifo or serial line.
//## we will keep the UNIX-Abstraction here
//## The following advantages:
//##   Modeling in UML
//##   Simpler programming with exceptions
//##   better tracing
class RACE_Device
{
public:
    RACE_Device();
    void open(const char *path,int flags) throw(Systemerror);
    //##Documentation
    //## simple read(2) call
    //## returns 0 if nothing read or if would block
    //## (avoid exception in this case)
    ssize_t read(char *buffer,size_t buflen) throw (Systemerror,SystemCallInterrupted);
    void close();
    static envdebug debug;
private:
    int fd_;
    const char *path_;
};
#endif
