// -*- C++ -*- -----------------------------------------------------
//  ----------------------------------------------------------------
//  --
//  -- Project            : SQ2000
//  -- SCU                : INTAPP
//  -- Subunit/Task       : INTAPP_NAV_GPS_Proxy
//  -- File               : $RCSfile: serialtest.cpp,v $
//  --
//  -- Prototype showing the working of the ACE-Reactor with the serial 
//  -- line
//  ----------------------------------------------------------------
//  ----------------------------------------------------------------
//  --   Initial Author       : Rudolf Weber ICS AG
//  --   Last changed by      : $Author: e000593d_intapp $
//  --   Date of last changes : $Date: 2005/04/18 15:10:20 $
//  --   reserved by          : $Locker:  $
//  --   State                : $State: Exp $
//  --
//  --====History=================================================
//  -- $Log: serialtest.cpp,v $
//  -- Revision 1.3  2005/04/18 15:10:20  e000593d_intapp
//  -- Compiles with RACE-Copy
//  --
//  -- Revision 1.2  2005/04/18 13:46:54  e000593d_intapp
//  -- Build in NMEA0183_Message_Assembler feeding a Buffer for a periodic Consumer-Thread
//  --
//  -- Revision 1.1.1.1  2005/04/08 07:06:13  e000593d_intapp
//  -- Prototype ACE_Reactor and serial lines 
//  --

#include<ace/streams.h> // more portable as #include<iostream>
#include<ace/Reactor.h>
#include"NMEA0183_Connector.h"
#include"SerialInputHandler.h"
#include"NMEA0183_Message_Assembler.h"
#include"SimpleProcessor.h"
#include"Consumer.h"
#include"RACE_PeriodicWorkerThread.h"

/*
   Problems: handle_close and destructor not called
*/
class TerminationHandler : public ACE_Event_Handler
{
public:
    TerminationHandler();
    virtual int handle_signal(int signum, siginfo_t * = 0, ucontext_t * = 0);
    virtual int handle_close (ACE_HANDLE handle,ACE_Reactor_Mask close_mask);
    virtual ~TerminationHandler();
    bool signalArrived();
private:
    bool m_arrived;
};

TerminationHandler::TerminationHandler():m_arrived(false)
{
}

int TerminationHandler::handle_signal(int signum, siginfo_t *, ucontext_t *)
{
    std::cerr << "Signal " << signum << " received" << std::endl;
    m_arrived=true;
    return 0;
}

int TerminationHandler::handle_close(ACE_HANDLE /*handle*/,ACE_Reactor_Mask /*close_mask*/)
{
    std::cerr << "handle_close" << std::endl;
    return 0;

}

bool TerminationHandler::signalArrived()
{
    return m_arrived;
}
  
TerminationHandler::~TerminationHandler()
{
    std::cerr << "TerminationHandler::~TerminationHandler()" << std::endl;
}


static ACE_TTY_IO serial_dev;

int main(int,char **)
{
    long output_periode_hz=10;
    ACE_Reactor *reactor=ACE_Reactor::instance();
    TerminationHandler *termhandler = new TerminationHandler();
    reactor->register_handler(SIGTERM,termhandler);
    reactor->register_handler(SIGINT,termhandler);
    NMEA0183_Connector connector;
    if(connector.connect(serial_dev,ACE_DEV_Addr("/dev/ttyS0"))<0)
    {
       ACE_ERROR_RETURN ((LM_ERROR,"%p\n","/dev/ttyS0"),-1);
    }
   
    SimpleBuffer *sb = new SimpleBuffer();
    SimpleProcessor *processor = new SimpleProcessor(sb);
    RACE::Message_Assembler *assembler = new NMEA0183_Message_Assembler(processor);
    SerialInputHandler *serialih = new SerialInputHandler(&serial_dev,assembler);
    reactor->register_handler(serialih,ACE_Event_Handler::READ_MASK);
    long timerid=reactor->schedule_timer(serialih,(const void *)0,ACE_Time_Value(2),ACE_Time_Value(2));
    serialih->settimerid(timerid);

    //consumer site
#if 0
       Consumer *consumer = new Consumer(ACE_Time_Value(0,1000000/output_periode_hz),sb);
#else       
    Consumer *consumer = new Consumer(ACE_Time_Value(2,0*output_periode_hz),sb);
#endif
    RACE::PeriodicWorkerThread *consumerthread=new RACE::PeriodicWorkerThread(consumer);
    std::cerr << "Periode of consumer thread: " << consumer->getPeriod().sec() 
	      << ':' << consumer->getPeriod().usec() << std::endl;
    consumerthread->start();
    // reactor->run_reactor_event_loop();
    // reactor->handle_events();  // only once
    while(!termhandler->signalArrived())
    {
	reactor->handle_events();
    }
    std::cerr << "properly shutdown " << std::endl;
    serial_dev.close();
    consumerthread->stop();
    RACE_Thread::wait_for_all_threads();
    return 0;
}
